from izug.ticketbox import ticketboxMessageFactory as _

def map_attribute(context, listname, id=None):
    """search the title-name from a list with the id
    """

    if listname == "priority":
        mapped_title = map_priority(context, id)
    elif listname == "state":
        mapped_title = map_state(context, id)
    elif listname == "area":
        mapped_title = map_area(context, id)
    elif listname == "variety":
        mapped_title = map_variety(context, id)
    elif listname == "releases":
        mapped_title = map_release(context, id)
    elif listname == "watchedRelease":
         mapped_title = map_watch_release(context, id)
    else:
        mapped_title = ""

    return mapped_title


def map_priority(context, id=None):
    """search the title-name of a list with the id
    """
    if not id and getattr(context, 'getPriority', None):
        id = context.getPriority()

    return map_base(context.getAvailablePriorities(), id)


# # HELPER Methods for ftw.table generator
def map_state(context, id=None):
    """search the title-name of a list with the id
    """

    if not id and getattr(context, 'getState', None):
        id = context.getState()
    return map_base(context.getAvailableStates(), id)


def map_area(context, id=None):
    """search the title-name of a list with the id
    """

    if not id and getattr(context, 'getArea', None):
        id = context.getArea()

    return map_base(context.getAvailableAreas(), id)


def map_variety(context, id=None):
    """search the title-name of a list with the id
    """
    if not id and getattr(context, 'getVariety', None):
        id = context.getVariety()

    return map_base(context.getAvailableVarieties(), id)


def map_release(context, id=None):
    """search the title-name of a list with the id
    """
    if not id and getattr(context, 'getReleases', None):
        id = context.getReleases()

    return map_base(context.getAvailableReleases(), id)


def map_watch_release(context, id=None):
    """search the title-name of a list with the id
    """
    if not id and getattr(context, 'getWatchedRelease', None):
        id = context.getWatchedRelease()

    return map_base(context.getAvailableReleases(), id)


def map_base(available_items, id):
     """Basemapping for attributes in ticketbox
     """

     for available_item in available_items:
         if id == available_item.get('id'):
             return available_item.get('title')
     return "-"


def readable_author(context):
    """
    get the full name of a user-id
    """
    author = context.getResponsibleManager()

    if not author:
        return '-'
    name = author
    user = context.acl_users.getUserById(author)
    if user is None:
        return _(u"unassigned")
    else:
        name = user.getProperty('fullname', author)
        if not len(name):
            name = author
    return name



import unittest
from izug.ticketbox.tests.base import TicketBoxTestCase
from zope.component import getMultiAdapter


class TestTicketBox(TicketBoxTestCase):

    def afterSetUp(self):
        super(TestTicketBox, self).afterSetUp()

    def beforeTearDown(self):
        pass

    def test_required_fields(self):
        title = self.ticketbox.getField('title')
        description = self.ticketbox.getField('description')

        self.assertEquals(title.required, True)
        self.assertEquals(description.required, True)

    def test_generated_id(self):
        state = self.ticketbox.getAvailableStates()[2]['id']
        area = self.ticketbox.getAvailableAreas()[2]['id']
        release = self.ticketbox.getAvailableReleases()[2]['id']
        priority = self.ticketbox.getAvailablePriorities()[2]['id']

        self.assertEquals(state, "test-id-state")
        self.assertEquals(area, "test-id-area")
        self.assertEquals(release, "test-id-release")
        self.assertEquals(priority, "test-id-priority")

    def test_overview_view(self):

        ticketbox_view = getMultiAdapter((self.ticketbox, self.portal.REQUEST), name='tabbedview_view-Overview')
        ticket_state = ticketbox_view.getFilteredTickets(state='test_id_1')
        ticket_area = ticketbox_view.getFilteredTickets(area='test_id_1')
        ticket_priority = ticketbox_view.getFilteredTickets(priority='test_id_1')
        ticket_responsible = ticketbox_view.getFilteredTickets(responsible='testuser1')

        self.assertEquals(ticket_state[0].getObject().getState(), "test_id_1")
        self.assertEquals(ticket_area[0].getObject().getArea(), "test_id_1")
        self.assertEquals(ticket_priority[0].getObject().getPriority(), "test_id_1")
        self.assertEquals(ticket_responsible[0].getObject().getResponsibleManager(), "testuser1")

    def test_all_tickets_view(self):

        ticketbox_view = getMultiAdapter((self.ticketbox, self.portal.REQUEST), name='tabbedview_view-all_tickets')
        ticketbox_view.search(kwargs={'portal_type':'Ticket'})

        self.assertEquals(ticketbox_view.len_results, 1)

    def test_my_tickets_view(self):

        ticketbox_view = getMultiAdapter((self.ticketbox, self.portal.REQUEST), name='tabbedview_view-my_tickets')
        ticketbox_view.search(kwargs={'portal_type':'Ticket'})

        self.assertEquals(ticketbox_view.len_results, 1)



def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestTicketBox))
    return suite



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import Schema, registerType
from Products.Archetypes.atapi import StringField, StringWidget
from Products.CMFCore.utils import getToolByName
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from ftw.tabbedview.interfaces import ITabbedView
from izug.ticketbox import ticketboxMessageFactory as _
from izug.ticketbox.config import PROJECTNAME
from izug.ticketbox.interfaces import ITicketBox
from zope.component import queryUtility, getUtility
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory


TicketBoxSchema = folder.ATBTreeFolderSchema.copy() + Schema((

        #Individual Identifier
        StringField(
            name='individualIdentifier',
            searchable=True,

            widget=StringWidget(
                label=_(u"label_individualIdentifier",
                        default=u"Individual identifier"),
                description=_(u"help_individualIdentifier",
                              default=u"Enter a individual identifier " + \
                                  u"(max 7 positions)"),
                maxlength=7,
                )),

        #Available States
        DataGridField(
            name='availableStates',
            searchable=True,
            allow_empty_rows=False,

            default=(
                {'id': '',
                 'title': "offen",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "in Bearbeitung",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "zur\xc3\xbcckgewiesen",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "zum Testen",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },

                {'id': '',
                 'title': "erledigt",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '0',
                 },

                {'id': '',
                 'title': "verschoben",
                 'show_in_all_tickets': '1',
                 'show_in_my_tickets': '1',
                 },
                ),
            columns=("id", "title",
                     "show_in_all_tickets", "show_in_my_tickets"),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u"Define states"),
                description=_(u"add or delete possible state-information"),
                columns={
                    'id': Column(_(u"id")),
                    'title': Column(_(u"title")),
                    'show_in_all_tickets': SelectColumn(
                        _(u"show in 'all tickets'"),
                        # DatagridFields SelectColumn doesn not behave like an
                        # archetype field. It always does
                        # a getattr to get the vocab.
                        vocabulary="yes_no"),
                    'show_in_my_tickets': SelectColumn(
                        _(u"show in 'my tickets'"),
                        vocabulary="yes_no"),
                    })),

        #Available Releases
        DataGridField(
            name='availableReleases',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u'label_availableReleases',
                        default=u"Available Releases"),
                description=_(u'help_avialableReleases',
                              default=u"Enter the Available Releases " + \
                                  u"for this tracker."),
                column_names=(_(u'Releases_id'), _(u'Releases_title')))),

        #Available Priorities
        DataGridField(
            name='availablePriorities',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u'label_availablePriorities',
                        u"Available priorities"),
                description=_(u'help_availablePriorities',
                               default=u"Enter the different type of " + \
                                  u"issue severities that should be " + \
                                  u"available, one per line."),
                column_names=(
                    _(u'Priorities_id'),
                    _(u'Priorities_title')))),

        #Available Areas
        DataGridField(
            name='availableAreas',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                visible={'view': 'invisible', 'edit': 'visible'},
                label=_(u'label_Areas', default=u"Areas"),
                description=_(u'help_areas',
                              default=u"Enter the issue topics/areas" + \
                                  u" for this tracker."),
                column_names=(
                    _(u'Areas_id'),
                    _(u'Areas_title')),
                )),

        #Available Varieties
        DataGridField(
            name='availableVarieties',
            allow_empty_rows=False,
            columns=('id', 'title'),

            widget=DataGridWidget(
                label=_(u'label_Varieties', default=u"Varieties"),
                description=_(u'help_varieties',
                              default=u"Enter the issue varieties for " + \
                                  u"this tracker."),
                column_names=(
                    _(u'Varieties_id'),
                    _(u'Varieties_title')))),

        ))


schemata.finalizeATCTSchema(
    TicketBoxSchema,
    folderish=True,
    moveDiscussion=False,
    )


class TicketBox(folder.ATBTreeFolder):
    """A tracker-like task management system"""

    implements(ITicketBox, ITabbedView)

    meta_type = "TicketBox"
    schema = TicketBoxSchema
    security = ClassSecurityInfo()

    def assignable_users(self):
        """
        Get the managers available as a DisplayList. The first item is 'None',
        with a key of '(UNASSIGNED)'.
        """

        factory = queryUtility(IVocabularyFactory, name='assignable_users')
        if factory is None:
            factory = getUtility(IVocabularyFactory,
                                 name='plone.principalsource.Users',
                                 context=self)

        vocabulary = factory(self)

        # TODO: Use vocabulary instead of list of tuples.
        users = [('(UNASSIGNED)', _(u'None'))]

        mtool = getToolByName(self, 'portal_membership')

        for term in vocabulary:
            member = mtool.getMemberById(term.token)
            if member and 'Contributor' in member.getRolesInContext(self):
                title = term.title
                if not title:
                    title = term.value
                users.append((term.token, title))

        return users

    def yes_no(self):
        """return displaylist with two static rows
        contents: yes and no
        """
        return DisplayList((
                ("1", _(u"yes")),
                ("0", _(u"no")),
                ))

registerType(TicketBox, PROJECTNAME)



"""Test setup for integration and functional tests.

When we import PloneTestCase and then call setupPloneSite(), all of Plone's
products are loaded, and a Plone site will be created. This happens at module
level, which makes it faster to run each test, but slows down test runner
startup.
"""

from Products.CMFCore.utils import getToolByName
from Products.Five import fiveconfigure
from Products.Five import zcml
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup
from Testing import ZopeTestCase as ztc
from izug.ticketbox.handlers import generate_datagrid_column_id
from izug.ticketbox.adapters import Response, ResponseContainer

# Mock data to be used in various tests in izug.ticketbox
MOCK_STATE = [{'id': 'test_id_1',
             'title': "Show All Tickets",
             'show_in_all_tickets': '1',
             'show_in_my_tickets': '0',
            },
            {'id': 'test_id_2',
             'title': "Show My Tickets",
             'show_in_all_tickets': '0',
             'show_in_my_tickets': '1',
            },
            {'id': '',
             'title': "Test Id State",
             'show_in_all_tickets': '1',
             'show_in_my_tickets': '1',
            },
            ]

MOCK_AREA = [{'id': u'test_id_1',
             'title': u'Internet',
             },
             {'id': u'test_id_2',
              'title': u'Intranet',
             },
             {'id': u'',
             'title': u'Test Id Area',
             },
            ]


MOCK_RELEASE = [{'id': u'version-1',
             'title': u'Version 1',
             },
             {'id': u'version-2',
              'title': u'Version 2',
             },
             {'id': u'',
             'title': u'Test Id Release',
             },
             ]


MOCK_PRIORITY = [{'id': u'test_id_1',
             'title': u'High',
             },
             {'id': u'test_id_2',
              'title': u'Low',
             },
             {'id': u'',
             'title': u'Test Id Priority',
             },
             ]


MOCK_RESPONSIBLE = [{'id': u'testuser',
             'title': u'Testuser',
             },
             {'id': u'admin',
              'title': u'Admin',
             },
             ]


#
# When ZopeTestCase configures Zope, it will *not* auto-load products in
# Products/. Instead, we have to use a statement such as:
#
#   ztc.installProduct('SimpleAttachment')
#
# This does *not* apply to products in eggs and Python packages (i.e. not in
# the Products.*) namespace. For that, see below.
#
# All of Plone's products are already set up by PloneTestCase.
#

@onsetup
def setup_product():
    """Set up the package and its dependencies.

    The @onsetup decorator causes the execution of this body to be deferred
    until the setup of the Plone site testing layer. We could have created our
    own layer, but this is the easiest way for Plone integration tests.
    """

    # Load the ZCML configuration for the example.tests package.
    # This can of course use <include /> to include other packages.
    fiveconfigure.debug_mode = True
    import izug.ticketbox
    zcml.load_config('configure.zcml', izug.ticketbox)
    fiveconfigure.debug_mode = False

    # We need to tell the testing framework that these products
    # should be available. This can't happen until after we have loaded
    # the ZCML. Thus, we do it here. Note the use of installPackage() instead
    # of installProduct().
    #
    # This is *only* necessary for packages outside the Products.* namespace
    # which are also declared as Zope 2 products, using
    # <five:registerPackage /> in ZCML.

    # We may also need to load dependencies, e.g.:
    #
    #   ztc.installPackage('borg.localrole')
    #

    ztc.installPackage('izug.ticketbox')

# The order here is important: We first call the (deferred) function which
# installs the products we need for this product. Then, we let PloneTestCase
# set up this product on installation.

if __name__ == '__main__':
    setup_product()
    ptc.setupPloneSite(products=['izug.ticketbox'])

class TicketBoxTestCase(ptc.PloneTestCase):

    def afterSetUp(self):
        # Set up sessioning objects
        ztc.utils.setupCoreSessions(self.app)

        self.workflow = getToolByName(self.portal, 'portal_workflow')
        self.acl_users = getToolByName(self.portal, 'acl_users')
        self.types = getToolByName(self.portal, 'portal_types')

        self.setRoles(('Manager',))

        #Create a Ticketbox on ploneroot
        self.portal.invokeFactory('Ticket Box', 'ticketbox')

        self.ticketbox = self.portal['ticketbox']
        self.ticketbox.getField('title').set(self.ticketbox, "Ticket Box Title")
        self.ticketbox.getField('description').set(self.ticketbox, "A TicketBox description")
        self.ticketbox.setIndividualIdentifier("ABC123")
        self.ticketbox.setAvailableStates(MOCK_STATE)
        self.ticketbox.setAvailableReleases(MOCK_RELEASE)
        self.ticketbox.setAvailableAreas(MOCK_AREA)
        self.ticketbox.setAvailablePriorities(MOCK_PRIORITY)

        #Generate Ids
        generate_datagrid_column_id(self.ticketbox, self)

        #Create a ticket on the ticketbox
        self.portal.ticketbox.invokeFactory('Ticket', 'ticket1')
        self.ticket1 = self.portal.ticketbox['ticket1']
        self.ticket1.getField('title').set(self.ticketbox, "Ticket Title")
        self.ticket1.getField('description').set(self.ticketbox, "A Ticket description")
        self.ticket1.getField('state').set(self.ticket1, MOCK_STATE[0]['id'])
        self.ticket1.getField('area').set(self.ticket1, MOCK_AREA[0]['id'])
        self.ticket1.getField('priority').set(self.ticket1, MOCK_PRIORITY[0]['id'])
        self.ticket1.getField('releases').set(self.ticket1, MOCK_RELEASE[0]['id'])
        self.ticket1.getField('responsibleManager').set(self.ticket1, "testuser1")
        self.ticket1.reindexObject()

        #Create a ticket on the ticketbox
        self.portal.ticketbox.invokeFactory('Ticket', 'ticket2')
        self.ticket2 = self.portal.ticketbox['ticket2']
        self.ticket2.getField('title').set(self.ticketbox, "Ticket Title")
        self.ticket2.getField('description').set(self.ticketbox, "A Ticket description")
        self.ticket2.getField('state').set(self.ticket2, MOCK_STATE[1]['id'])
        self.ticket2.getField('area').set(self.ticket2, MOCK_AREA[1]['id'])
        self.ticket2.getField('priority').set(self.ticket2, MOCK_PRIORITY[1]['id'])
        self.ticket2.getField('releases').set(self.ticket2, MOCK_RELEASE[1]['id'])
        self.ticket2.getField('responsibleManager').set(self.ticket2, "testuser1")
        self.ticket2.reindexObject()

        #Create responsecontainers
        self.container1 = ResponseContainer(self.ticket1)
        self.container2 = ResponseContainer(self.ticket2)

        #Create responses
        self.response1 = Response("response1")
        self.response2 = Response("response3")
        self.response3 = Response("response3")

class FunctionalTestCase(ptc.FunctionalTestCase):
    """We use this class for functional integration tests that use doctest
    syntax. Again, we can put basic common utility or setup code in here.
    """



from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
from DateTime import DateTime
from izug.ticketbox import ticketboxMessageFactory as _
from izug.ticketbox.config import PROJECTNAME
from izug.ticketbox.interfaces import ITicket
from Products.Archetypes.atapi import AttributeStorage
from Products.Archetypes.atapi import DateTimeField, CalendarWidget
from Products.Archetypes.atapi import FileField, FileWidget
from Products.Archetypes.atapi import ReferenceField
from Products.Archetypes.atapi import registerType
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import StringField
from Products.ATContentTypes.content import base
from Products.ATContentTypes.content import schemata
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget \
    import ReferenceBrowserWidget
from zope.interface import implements
from Products.Archetypes import atapi
from Products.CMFCore.permissions import ManagePortal

TicketSchema = schemata.ATContentTypeSchema.copy() + Schema((

    #Due-Date (default: x + 14 days)
    DateTimeField(
        name='dueDate',
        default_method='default_due_date',
        required=True,
        validators=('isValidDate'),
        widget=CalendarWidget(
            label=_(u"label_duedate", default=u"Due Date"),
            description=_(u"help_duedate",default=u"Due-date of the ticket"),
        )
    ),

    #State
    StringField(
        name='state',
        vocabulary_factory='ticketbox_values_states',
        widget=SelectionWidget(
            format="select",
            label=_(u"label_state", default=u"State"),
            description=_(u"help_state",default=u"Define Which State the Ticket has"),
        ),
    ),

    #Priority
    StringField(
        name='priority',
        vocabulary_factory='ticketbox_values_priorities',
        widget=SelectionWidget(
            format="select",
            label=_(u"label_select_priority",default=u"Select Priority"),
            description=_(u"help_priority", default=u"Select the Priority"),
        ),
    ),

    #Area
    StringField(
        name='area',
        vocabulary_factory='ticketbox_values_areas',
        widget=SelectionWidget(
            format="select",
            label=_(u"label_select_area",default=u'Select Area'),
            description=_(u'help_area',default=u'Select the Area of the ticket'),
        ),
    ),

    #Variety
    StringField(name='variety',
                vocabulary_factory='ticketbox_values_varieties',

                widget=SelectionWidget(
                        label=_(u"label_select_variety",
                                default=u'Select Variety'),
                        description=_(u'help_variety',
                                      default=u'Select the Variety of the ticket'),
                        format="select")),

    #Releases
    # XXX: RENAME TO RELEASE
    StringField(
        name='releases',
        vocabulary_factory='ticketbox_values_releases',
        widget=SelectionWidget(
            format="select",
            label=_(u'label_select_release', default=u'Select Release'),
            description=_(u'help_release', default=u'Select the Release of the ticket'),
        ),
    ),

    #Watched in release
    StringField(
            name='watchedRelease',
            vocabulary_factory='ticketbox_values_releases',
            widget=SelectionWidget(
                format="select",
                label=_(u'label_select_watchedRelease',
                        default=u'Select watched in release'),
                description=_(u'help_watchedRelease',
                              default=u'Select in which release you watched it'),
                )),

    #Responsible
    StringField(
        name='responsibleManager',
        index="FieldIndex:schema",
        widget=SelectionWidget(
            format="select",
            label=_(u'label_Responsible',default=u"Responsible"),
            description=_(u'help_responsible',default=u"Select which manager, if any," +
                " is responsible for this issue."),
        ),
        vocabulary='assignable_users',
    ),

    #Answer-date (default: x + 14 days)
    DateTimeField(
        name='answerDate',
        default_method='default_answer_date',
        widget=CalendarWidget(
            label=_(u'label_AnswerDate',default=u"Answerdate"),
            description=_(u'help_answerdated',default=u"Answer-date of the ticket"),
        )
    ),

    #Attachment
    FileField(
        name='attachment',
        widget=FileWidget(
            label=_(u'label_attachment',default=u"Attachment"),
            description=_(u'help_attachment',default=u"You may optionally upload a file attachment." +
                " Please do not upload unnecessarily large files."),
        ),
        storage=AttributeStorage(),
    ),

    #References for Atachments
    ReferenceField(
        name='attachments',
        widget=ReferenceBrowserWidget(
            label=_(u'label_attachments', default=u"Attachments"),
            allow_browse=True,
            show_results_without_query=True,
            restrict_browsing_to_startup_directory=True,
            base_query={
                "portal_type": "TicketAttachment",
                "sort_on": "sortable_title"},
            visible={'view': 'visible', 'edit': 'invisible'},
        ),
        allowed_types=('TicketAttachment'),
        multiValued=1,
        schemata='default',
        relationship='TicketAttachment'
    ),

    #References
    ReferenceField(
        name='ticketReferences',
        widget=ReferenceBrowserWidget(
            label=_(u'label_references',default=u"References"),
            allow_browse=True,
            show_results_without_query = False,
            restrict_browsing_to_startup_directory=False,

        ),
        multiValued=1,
        schemata='default',
        relationship='TicketBox'
    ),
))

TicketSchema['description'].widget = atapi.RichWidget(
    label=_(u"label_description",
    default=u"Description"),
    rows=30)
schemata.finalizeATCTSchema(TicketSchema, moveDiscussion=False)

# Hide all unimportant fields except default-schamata-fields
for field in TicketSchema.keys():
    if TicketSchema[field].schemata != 'default':
        TicketSchema[field].write_permission = ManagePortal

class Ticket(base.ATCTFolder):
    """A ticket for a tracker-like task management system"""
    implements(ITicket)

    security = ClassSecurityInfo()
    meta_type = "Ticket"
    schema = TicketSchema

    def generateNewId(self):
        """generate a new ticket id.
        get all tickets from parent (ticketbox)
        and looks for the higherst id and add one to be unique
        """
        parent = self.aq_parent
        maxId = 0
        for id in parent.objectIds():
            try:
                intId = int(id)
                maxId = max(maxId, intId)
            except (TypeError, ValueError):
                pass
        return str(maxId + 1)

    def default_due_date(self):
        """ Return a default due-date (9.00 Clock in 14 days) """

        a = DateTime() + 14
        return DateTime(a.year(), a.month(), a.day(), 9, 00)

    def default_answer_date(self):
        """ Return a default answer-date (9.00 Clock in 14 days) """

        return self.default_due_date()

    def assignable_users(self):
        """
        Get the managers available as a DisplayList. The first item is 'None',
        with a key of '(UNASSIGNED)'.
        """

        return aq_parent(self).assignable_users()


registerType(Ticket, PROJECTNAME)



from izug.ticketbox.browser.helper import map_attribute, readable_author
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class TicketView(BrowserView):

    template = ViewPageTemplateFile('ticket_view.pt')

    # HELPER Methods for ftw.table generator
    def map_state(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "state")

    def map_priority(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "priority")

    def map_area(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "area")

    def map_variety(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "variety")

    def map_release(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "releases")

    def map_watched_release(self):
        """
        search the title-name of a list with the id
        """
        return map_attribute(self.context, "watchedRelease")

    def map_author(self):
        """
        get the full name of a user-id
        """
        return readable_author(self.context)



from ftw.notification.email.templates.base import BaseEmailRepresentation
from zope.app.pagetemplate import ViewPageTemplateFile
from izug.ticketbox.browser.helper import map_attribute, readable_author
from izug.ticketbox import ticketboxMessageFactory as _


class TicketEmailRepresentation(BaseEmailRepresentation):

    template = ViewPageTemplateFile('ticket.pt')

    def getTracker(self):
        context = self.context
        tracker = context.aq_parent

        return tracker

    def creator(self):
        context = self.context.aq_inner
        return context.Creator()

    def infos(self):
        """Returns Infos for email-template"""
        base_response = self.context.restrictedTraverse('base_response')
        responses = base_response.responses()
        author = self.context.translate(readable_author(self.context))

        ticket_infos = {'tracker_title': self.context.aq_parent.title,
                    'tracker_url': self.context.aq_parent.absolute_url(),
                    'title': self.context.Title(),
                    'ticket_id': self.context.getId(),
                    'individualIdendifier':
                    self.context.aq_parent.getIndividualIdentifier,
                    'url': self.context.absolute_url(),
                    'text': self.context.Description(),
                    'state': map_attribute(self.context, "state"),
                    'responsibleManager': author,
                    'priority': map_attribute(self.context, "priority"),
                    'area': map_attribute(self.context, "area"),
                    'variety': map_attribute(self.context, "variety"),
                    'releases': map_attribute(self.context, "releases"),
                    'watchedRelease': map_attribute(self.context, "watchedRelease"),
                    'answerDate':self.context.getAnswerDate(),
                    'response': False}
        if responses == []:
            return ticket_infos
        else:
            response_date = responses[len(responses)-1]['response'].date
            if self.context.modification_date > response_date:
                return ticket_infos
            else:
                response = responses[len(responses)-1]
                changes = {
                    'tracker_title': self.context.aq_parent.title,
                    'tracker_url': self.context.aq_parent.absolute_url(),
                    'title': self.context.Title(),
                    'ticket_id': self.context.getId(),
                    'individualIdendifier':
                        self.context.aq_parent.getIndividualIdentifier,
                    'url': self.context.absolute_url(),
                    'text': '',
                    'state': '',
                    'responsibleManager': '',
                    'priority': '',
                    'area': '',
                    'variety': '',
                    'releases': '',
                    'watchedRelease': '',
                    'answerDate':'',
                    'response': True}
                for item in response['response'].changes:
                    # XXX: Hack to solve the label_unassigned translations problem
                    # If we retrieve a responsibleManager named label_unassigned,
                    # try to translate it
                    if item['id'] == 'responsibleManager':
                        if item['before'] == 'label_unassigned':
                            item['before'] = self.context.translate(_(item['before']))
                        if item['after'] == 'label_unassigned':
                            item['after'] = self.context.translate(_(item['after']))

                    changes[item['id']] = (
                        item['before'] + 	' &rarr; ' + item['after'])
                changes['text'] = response['response'].text
                return changes



"""Common configuration constants
"""

PROJECTNAME = 'izug.ticketbox'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'Ticket': 'izug.ticketbox: Add Ticket',
    'TicketBox': 'izug.ticketbox: Add Ticket Box',
}
INDEXES = (('getState', 'FieldIndex'),
            ('getReleases', 'FieldIndex'),
            ('getWatchedRelease', 'FieldIndex'),
            ('getArea', 'FieldIndex'),
            ('getVariety', 'FieldIndex'),
            ('getPriority', 'FieldIndex'),
            ('getResponsibleManager', 'FieldIndex'),
            ('getDueDate', 'DateIndex'),
            ('sortable_id', 'FieldIndex'),
            ('sortable_responsibleManager', 'FieldIndex'),
            ('sortable_ticket_references', 'FieldIndex'),
            ('get_owner_index', 'FieldIndex'),
           )

# Add text/html to the list of mimetypes to allow HTML/kupu
# issue/response text.
ISSUE_MIME_TYPES = ('text/html', )
DEFAULT_ISSUE_MIME_TYPE = 'text/html'



from Products.Five.browser import BrowserView
from izug.ticketbox import ticketboxMessageFactory as _


def format_date(date):
    if not date:
        return ''
    if date.year() <= 1900:
        return ''
    else:
        return date.strftime('%d.%m.%Y %H:%m')


def map_base(available_items, id_):
    """Basemapping for attributes in ticketbox
    """

    for available_item in available_items:
        if id_ == available_item.get('id'):
            return available_item.get('title')
    return "-"


class TicketsExport(BrowserView):
    """Returns a csv wincp52 ecoded file"""

    def fullname(self, uid):
        member = self.context.portal_membership.getMemberById(uid)
        if member:
            fullname = member.getProperty('fullname', member.getId())
            if fullname:
                return fullname
            return member.getId()
        return uid

    def __call__(self):

        response = self.request.RESPONSE
        context = self.context
        filename = 'export_%s.csv' % self.context.getId()
        encoding = 'Windows-1252'
        csv = []

        brains = list(context.portal_catalog({
            'portal_type': 'Ticket',
            'sort_on': 'getId',
            'path': {
                'query': '/'.join(context.getPhysicalPath()), 'depth': 2}}))

        headers = [
            context.translate(_(u'csv_heading_nr', default=u'Nr.')),
            context.translate(_(u'csv_heading_title', default=u'Title')),
            context.translate(
                _(u'csv_heading_description', default=u'Description')),
            context.translate(_(u'csv_heading_creator', default=u'Creator')),
            context.translate(_(u'csv_heading_created', default=u'Created')),
            context.translate(
                _(u'csv_heading_responsible', default=u'Responsible')),
            context.translate(_(u'csv_heading_state', default=u'State')),
            context.translate(
                _(u'csv_heading_priorities', default=u'Priorities')),
            context.translate(_(u'csv_heading_area', default=u'Area')),
            context.translate(_(u'csv_heading_variety', default=u'Variety')),
            context.translate(_(u'csv_heading_releases', default=u'Releases')),
            context.translate(
                _(u'csv_heading_watched_release', default=u'Watched release')),
            context.translate(
                _(u'csv_heading_watched_duedate', default=u'Duedate')),
            ]

        # Header
        csv.append(','.join(
            [field.encode('utf-8') for field in headers]))

        # Data
        for brain in brains:
            row = []
            row.append(brain.getId)
            row.append(brain.Title)
            row.append(brain.Description)
            row.append(self.fullname(brain.Creator))
            row.append(format_date(brain.created))
            row.append(self.fullname(brain.getResponsibleManager))
            row.append(map_base(
                context.getAvailableStates(), brain.getState))
            row.append(map_base(
                context.getAvailablePriorities(), brain.getPriority))
            row.append(map_base(
                context.getAvailableAreas(), brain.getArea))
            row.append(map_base(
                context.getAvailableVarieties(), brain.getVariety))
            row.append(map_base(
                context.getAvailableReleases(), brain.getReleases))
            row.append(map_base(
                context.getAvailableReleases(), brain.getWatchedRelease))
            row.append(format_date(brain.getDueDate))

            # Change double quote to single
            row = [val.replace('"', "'") for val in row]

            # Put every value in double quote
            row = ['"%s"' % val for val in row]
            csv.append(','.join(row))
        result = '\n'.join(csv).decode('utf-8').encode(encoding, 'replace')
        response.setHeader('Content-Type',
                           'text/csv; charset=%s' % encoding)
        response.setHeader(
            'Content-disposition',
            'attachment; %s; filename=%s' % (encoding, filename))
        return result



from Acquisition import aq_parent
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary



class TicketBoxVocabulary(object):

    implements(IVocabularyFactory)

    def __init__(self, field):
        self.field = field
        super(TicketBoxVocabulary, self).__init__

    def __call__(self, context):
        tracker = aq_parent(context)
        myField = tracker.getField(self.field)
        result=[]
        for item in myField.get(tracker):
            result.append(SimpleVocabulary.createTerm(
            item['id'],item['id'],item['title']))
        if result == []:
            result.append(SimpleVocabulary.createTerm(
            '','','-'))
        return result


StatesVocabularyFactory=TicketBoxVocabulary('availableStates')
ReleasesVocabularyFactory=TicketBoxVocabulary('availableReleases')
PrioritiesVocabularyFactory=TicketBoxVocabulary('availablePriorities')
AreasVocabularyFactory=TicketBoxVocabulary('availableAreas')
VarietiesVocabularyFactory=TicketBoxVocabulary('availableVarieties')



import unittest
from izug.ticketbox.tests.base import TicketBoxTestCase
from DateTime import DateTime


class TestTicket(TicketBoxTestCase):

    def afterSetUp(self):
        super(TestTicket, self).afterSetUp()

    def beforeTearDown(self):
        pass

    def test_required_fields(self):
        title = self.ticket1.getField('title')
        description = self.ticket1.getField('description')

        self.assertEquals(title.required, True)
        self.assertEquals(description.required, True)

    def test_dates(self):
        due_date = self.ticket1.getField('dueDate').get(self.ticket1)
        answer_date = self.ticket1.getField('answerDate').get(self.ticket1)
        date = DateTime()+14

        self.assertEquals(due_date.Date(), date.Date())
        self.assertEquals(answer_date.Date(), date.Date())


def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestTicket))
    return suite



import unittest
from izug.ticketbox.tests.base import TicketBoxTestCase

import Products.Archetypes.Schema

class TestResponse(TicketBoxTestCase):

    def afterSetUp(self):
        super(TestResponse, self).afterSetUp()

    def beforeTearDown(self):
        pass

    def test_add_and_remove(self):

        self.assertEquals(len(self.container1), 0)
        self.assertEquals(len(self.container2), 0)

        self.container1.add(self.response1)
        self.container1.add(self.response2)
        self.container2.add(self.response3)

        self.assertEquals(len(self.container1), 2)
        self.assertEquals(len(self.container2), 1)

        self.container1.remove(0)
        self.container1.remove(1)

        self.assertEquals(self.response1 in self.container1, False)
        self.assertEquals(self.response2 in self.container1, False)
        self.assertEquals(self.response3 in self.container2, True)


def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestResponse))
    return suite



from setuptools import setup, find_packages
import os

version = '4.3.2'
maintainer = 'Elio Schmutz'

tests_require = [
    'zope.testing',
    'Products.PloneTestCase',
    ]

setup(name='izug.ticketbox',
      version=version,
      description="A tracker-like task management system for plone.",
      long_description=(open('README.rst').read() + '\n' + \
                            open(os.path.join('docs', 'HISTORY.txt')).read()),

      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.0',
        'Framework :: Plone :: 4.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Topic :: Software Development :: Libraries :: Python Modules', ],

      keywords='ticketbox tracker ftw ',
      author='4teamwork GmbH',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/izug.ticketbox',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['izug', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'setuptools',
        'Products.DataGridField',
        'ftw.tabbedview',
        'ftw.table',
        'ftw.notification.base',
        'ftw.notification.email',
        'plone.principalsource',
        'BeautifulSoup'
        ],

      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      test_suite='izug.ticketbox.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )




# # -*- coding: utf-8 -*-
from izug.ticketbox import ticketboxMessageFactory as _
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import queryUtility


def generate_datagrid_column_id(obj, event):
    """Generate column-ids for all datagrids in the ticketbox."""

    datagrid = []

    #save datagrid to change ids
    datagrid.append(obj.getAvailableStates())
    datagrid.append(obj.getAvailableReleases())
    datagrid.append(obj.getAvailablePriorities())
    datagrid.append(obj.getAvailableAreas())
    datagrid.append(obj.getAvailableVarieties())

    #change id from datagrids
    for dg in datagrid:
        for row in dg:
            if not row['id']:
                name = row['title']
                row['id'] = queryUtility(IIDNormalizer).normalize(name.decode('utf-8'))


def move_document_to_reference(obj, event):
    """Create own File and add it to References"""

    file_ = obj.getAttachment()
    if file_.data != '':
        new_id = queryUtility(IIDNormalizer).normalize(file_.filename.decode('utf-8'))
        if obj.get(new_id, None):
            IStatusMessage(obj.REQUEST).addStatusMessage(
                _(u"text_file_exists_error"), type='error')
            obj.setAttachment('DELETE_FILE')
            return
        new_file_id = obj.invokeFactory(
            type_name="TicketAttachment",
            id=new_id,
            title=file_.filename,
            file=file_)
        new_file = obj.get(new_file_id, None)
        if new_file is None:
            return
        uid = new_file.UID()
        references = obj.getRawAttachments()

        if not isinstance(references, list):
            references = [references]

        references.append(uid)
        obj.setAttachments(references)
        obj.setAttachment('DELETE_FILE')
        obj.reindexObject()
